from django import forms
from django.forms import widgets
from django.utils.encoding import force_text
from django.utils.safestring import mark_safe
from django.forms.utils import flatatt

from .util import FormControlMixin, TemplatedWidget


class CheckboxInput(TemplatedWidget, forms.CheckboxInput):
    template_name = 'django_bootstrap3_form/checkbox_widget.html'
    renders_label = True

    def get_context(self, name, value, attrs, label=None):
        context = super(CheckboxInput, self).get_context(name, value, attrs, label)

        context['checked'] = self.check_test(value)
        context['label'] = label

        if not (value is True or value is False or value is None or value == ''):
            context['value'] = force_text(value)

        if attrs:
            context['attrs'] = mark_safe(flatatt(attrs))

        return context


class PasswordInput(FormControlMixin, forms.PasswordInput):
    pass


class TextInput(FormControlMixin, forms.TextInput):
    pass


class Textarea(FormControlMixin, forms.Textarea):
    pass


class Select(FormControlMixin, forms.Select):
    pass


class RadioSelect(TemplatedWidget, forms.RadioSelect):
    template_name = 'django_bootstrap3_form/radio_select_widget.html'
    renders_label = True

    def get_context(self, name, value, attrs, label=None):
        context = super(RadioSelect, self).get_context(name, value, attrs, label)

        if value is None:
            value = ''

        context.update(
            selected_value=force_text(value),
            attrs=self.build_attrs(attrs),
            options=self.get_options(value),
        )

        return context

    def get_options(self, selected_value):
        selected_value = force_text(selected_value)

        def get_option(value, title):
            value = force_text(value)

            return dict(
                value=value,
                title=title,
                checked=(value == selected_value),
            )

        return [
            get_option(value, title) for value, title in self.choices
        ]


class DateInput(FormControlMixin, widgets.DateInput):
    input_type = 'date'

    def __init__(self, attrs=None, format_=None):
        # Html 5 date input expects this format
        format_ = format_ or '%Y-%m-%d'

        widgets.DateInput.__init__(self, attrs, format_)


class NumberInput(TextInput):
    input_type = 'number'



from django.template.loader import get_template


class FormControlMixin(object):
    css_class = 'form-control'

    def render(self, name, value, attrs=None, renderer=None):
        return super(FormControlMixin, self).render(name, value, self._get_attrs(attrs), renderer)

    def _get_attrs(self, attrs_param):
        def get_param():
            if attrs_param:
                return dict(attrs_param)
            else:
                return dict()

        attrs = get_param()

        class_string = self._get_class_string(attrs)

        if class_string:
            attrs['class'] = class_string

        return attrs

    def _get_class_string(self, attrs):
        classes = []
        if 'class' in attrs:
            classes.append(attrs['class'])

        classes += get_list(self.css_class)

        return ' '.join(classes)


def get_list(v):
    if not v:
        return []
    elif isinstance(v, (list, tuple)):
        return v
    else:
        return [v]


class TemplatedWidget(object):
    template_name = None

    def render(self, name, value, attrs=None, label=None):
        template = self.get_template()
        context = self.get_context(name, value, attrs, label)

        return template.render(context)

    def get_context(self, name, value, attrs, label=None):
        return dict(
            name=name,
            value=value,
            attr=attrs,
            label=label,
        )

    def get_template(self):
        if not hasattr(self, '_template'):
            self._template = get_template(self.template_name)

        return self._template



from django import forms

from django_pony_forms.pony_forms import PonyFormMixin


class BootstrapFormMixin(PonyFormMixin):
    form_template = 'django_bootstrap3_form/form.html'
    row_template = 'django_bootstrap3_form/row.html'
    errorlist_template = 'django_bootstrap3_form/errorlist.html'


class BootstrapForm(BootstrapFormMixin, forms.Form):
    pass


from django import forms

from .widgets import TextInput, CheckboxInput, Select, DateInput


class BooleanField(forms.BooleanField):
    widget = CheckboxInput

    def __init__(self, *args, **kwargs):
        self.autofocus = kwargs.pop('autofocus', None)

        super(BooleanField, self).__init__(*args, **kwargs)

    def widget_attrs(self, widget):
        attrs = super(BooleanField, self).widget_attrs(widget)

        if self.autofocus:
            attrs['autofocus'] = self.autofocus

        return attrs


class CharField(forms.CharField):
    widget = TextInput

    def __init__(self, *args, **kwargs):
        self.placeholder = kwargs.pop('placeholder', None)
        self.autofocus = kwargs.pop('autofocus', None)

        super(CharField, self).__init__(*args, **kwargs)

    def widget_attrs(self, widget):
        attrs = super(CharField, self).widget_attrs(widget)

        if self.placeholder:
            attrs['placeholder'] = self.placeholder

        if self.autofocus:
            attrs['autofocus'] = self.autofocus

        return attrs


class ChoiceField(forms.ChoiceField):
    widget = Select

    def __init__(self, *args, **kwargs):
        self.autofocus = kwargs.pop('autofocus', None)

        super(ChoiceField, self).__init__(*args, **kwargs)

    def widget_attrs(self, widget):
        attrs = super(ChoiceField, self).widget_attrs(widget)

        if self.autofocus:
            attrs['autofocus'] = self.autofocus

        return attrs


class DateField(forms.DateField):
    widget = DateInput


from setuptools import setup, find_packages

from os import path
from io import open

this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'readme.md'), encoding='utf-8') as f:
    long_description = f.read()

version = '0.7.0'

setup(
    name='django-bootstrap3-form',
    version=version,
    license='Apache License, Version 2.0',
    long_description=long_description,
    long_description_content_type='text/markdown',
    description='Django-bootstrap3-form allows you write Django forms that work with Twitter Bootstrap 3',
    packages=find_packages(exclude=['testproject', 'testproject.*']),
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'Django',
        'django_pony_forms',
        'six'
    ],
    author='Marco Braak',
    author_email='mbraak@ridethepony.nl',
    url='https://github.com/mbraak/django-bootstrap3-form',
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Framework :: Django",
        "Framework :: Django :: 1.11",
        "Framework :: Django :: 2.0",
        "Framework :: Django :: 2.1",
        "Framework :: Django :: 2.2",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7"
    ]
)
